#!/bin/bash
# Compiz/Compcomm get-git
# Copyright (c) 2007 Kristian Lyngstøl <kristian@bohemians.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
# This script attempts to get all fusion/compiz sources you need,
# build them and install them.

SUPRESS="true" ## set to false to display compile-stuff.

# Needed for premerge plugins
ITEMS="fusion/libraries/bcop" 
# The beryl plugins, ported to compiz
ITEMS="$ITEMS fusion/plugins-main fusion/plugins-extra"
# Emerald window decorator
ITEMS="$ITEMS fusion/decorators/emerald fusion/decorators/emerald-themes"

### CCS
# (formerly libbs), you can safly comment out this and use ini or gconf
ITEMS="$ITEMS fusion/compizconfig/libcompizconfig"
ITEMS="$ITEMS fusion/compizconfig/compizconfig-python"
ITEMS="$ITEMS fusion/compizconfig/ccsm"

# Uncomment one of these to use gconf or kconf backend for ccs
#ITEMS="$ITEMS fusion/compiz-configuration-system/libraries/ccs-backend-gconf"
#ITEMS="$ITEMS fusion/compiz-configuration-system/libraries/ccs-backend-kconfig"

#ITEMS="$ITEMS fusion/compiz-configuration-system/tools/ccs-settings fusion/compiz-configuration-system/bindings/ccs-python"

### CCS ends here

# My (==Kristian's) compiz-scripts. Includes this script and a compiz-manager 
# to start compiz and a decorator. Feedback on these scripts are much 
# appreaciated.

ITEMS="$ITEMS users/kristian/compiz-scripts"

# Add aditional items here if you need them.

######
# Items that won't be run with sudo make install, just make install.
# Suitable for plugins etc

ITEMS_NON_SUDO="users/kristian/zoom"

# Repository
REPO="git://anongit.opencompositing.org/"
# For developers, you may have to add your username.
#REPO="git+ssh://git.opencompositing.org/git/" 

# Compiz repo. "compiz" is appended.
COMPIZ="git://git.freedesktop.org/git/xorg/app/"

# Arguments for compiz' autogen.sh.  You shouldn't need this. I do.
COMPIZARGS="--enable-librsvg --enable-gtk --enable-metacity"

# Uncomment and set to your preffered location if you want a different
# prefix. Or export it before starting this script.
# PREFIX="/dev/random"

# Add arguments to pass to autogen.sh here. Note that this is NOT passed
# when building the core compiz package.
GENERICARGS=""

###############################################################
## No further tinkering should be necesarry beyond this comment

DEFPREFIX="/usr/local"
MYPREFIX=${PREFIX:-$DEFPREFIX}

# Check for git
if [ -z "$(which git-clone)" ]; then
	echo "You must have GIT installed to use this script. Install the git-core package."
	exit 1;
fi
if [ -z "$(which git-pull)" ]; then
	echo "You must have GIT installed to use this script. Install the git-core package."
	exit 1;
fi

# Asks if we want to bail out.
function bail_ask
{
	echo "$1"
	echo  -n "Abort?"
	read -p " [Y/n]: " -n1 continue;
	if [ "$(echo $continue | tr [:upper:] [:lower:])" = "y" -o "$continue" = "" ]; then
		echo 
		exit 3;
	fi
	echo
}

# Checks if a local directory for $1 exist (last part of it), 
# if it does; attempts to git-pull, otherwise git clone.
function get_repo
{
	b=`echo $1 | sed 's/.*\///g'`
	here=$PWD
	if [ -d $b ]; then
		cd $b
		echo "Updating $b";
		if [ $SUPRESS = "true" ]; then
			git-pull > /dev/null
		else
			git-pull
		fi
		cd $here
	else
		echo "Cloning $b";
		git-clone $2$1
	fi
}

# Get compiz
get_repo "compiz" $COMPIZ 

# Get everything else
for a in $ITEMS ; do
	get_repo $a $REPO
done
for a in $ITEMS_NON_SUDO ; do
	get_repo $a $REPO
done

# Builds $1 if autogen.sh exist.
function build 
{
	if [ ! -f $1/autogen.sh -a ! -f $1/Makefile ]; then
		echo "Don't know how to build $1 (no autogen.sh or Makefile). Skipping.";
		return;
	fi
	if [ ! -d $1 ]; then
		echo "$1 isn't a directory. Bailing out.";
		return 1;
	fi
	OLDPWD=$PWD
	cd $1
	if [ -f autogen.sh ]; then
	    echo "Building $1: ./autogen.sh $2"
	    if [ $SUPRESS = "true" ]; then
			./autogen.sh $2 > /dev/null
	    else
		  	./autogen.sh $2
	    fi
	    if [ $? -ne 0 ]; then
			echo "$1's autogen.sh reports errors. Bailing.";
			cd $OLDPWD
			return 1
	    fi
	fi

	echo "Building $1: make"
	if [ $SUPRESS = "true" ]; then
		make > /dev/null
	else
		make
	fi
	if [ $? -ne 0 ]; then 
		echo "$1's make returned errors. Bailing.";
		cd $OLDPWD
		return 1
	fi

	echo "Building $1: $3 make install"
	if [ $SUPRESS = "true" ]; then
		$3 make install > /dev/null
	else
		$3 make install
	fi
	if [ $? -ne 0 ]; then
		echo "$1's sudo make install returned an error and might not work.";
		cd $OLDPWD
		return 1
	fi
	cd $OLDPWD
}

# Build compiz first, and no need to continue if it fails.
if ! build compiz "$COMPIZARGS --prefix=$MYPREFIX" "sudo"; then
	echo "Failed to build compiz. No reason to continue.";
	exit 2;
fi

# Build everything else now.
for a in $ITEMS; do
	b=`echo $a | sed 's/.*\///g'`
	if ! build $b "$GENERICARGS --prefix=$MYPREFIX" "sudo"; then 
		bail_ask "Failed to build $b !"
	fi
done
for a in $ITEMS_NON_SUDO; do
	b=`echo $a | sed 's/.*\///g'`
	if ! build $b "$GENERICARGS --prefix=$MYPREFIX" ""; then 
		bail_ask "Failed to build $b !"
	fi
done
